package geniusweb.protocol.session.shaop;

import java.util.Arrays;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import geniusweb.actions.Comparison;
import geniusweb.protocol.session.TeamOfPartiesAndProfiles;
import geniusweb.references.PartyWithProfile;

/**
 * A tuple of a SHOAP party and a COB party. Part of the settings.
 *
 */
@JsonAutoDetect(fieldVisibility = Visibility.ANY, getterVisibility = Visibility.NONE, setterVisibility = Visibility.NONE)
public class ShaopTeam implements TeamOfPartiesAndProfiles {
	private final PartyWithProfile shaop;
	private final PartyWithProfile cob;

	/**
	 * @param shaop {@link PartyWithProfile} participants supporting SHAOP
	 *              behaviour. SHAOP behaviour has parameters:
	 *              <ul>
	 *              <li>elicitationcost: the value must be a double number
	 *              indicating the costs of each RequestComparison action.
	 *              <li>credit: the credit the party has to start with. The sum
	 *              of the elicitationcost should stay below this.
	 *              </ul>
	 * @param cob   {@link PartyWithProfile} party, must support the COB
	 *              behaviour. This is the cob partie that the shaop requests
	 *              for {@link Comparison} will be forwarded to
	 */
	@JsonCreator
	public ShaopTeam(@JsonProperty("shoap") PartyWithProfile shaop,
			@JsonProperty("cob") PartyWithProfile cob) {
		this.shaop = shaop;
		this.cob = cob;
	}

	@Override
	public List<PartyWithProfile> getAllParties() {
		return Arrays.asList(shaop, cob);
	}

	@Override
	public String toString() {
		return "Parties[" + shaop + "," + cob + "]";
	}

	public PartyWithProfile getShaop() {
		return shaop;
	}

	public PartyWithProfile getCob() {
		return cob;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((cob == null) ? 0 : cob.hashCode());
		result = prime * result + ((shaop == null) ? 0 : shaop.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ShaopTeam other = (ShaopTeam) obj;
		if (cob == null) {
			if (other.cob != null)
				return false;
		} else if (!cob.equals(other.cob))
			return false;
		if (shaop == null) {
			if (other.shaop != null)
				return false;
		} else if (!shaop.equals(other.shaop))
			return false;
		return true;
	}

}
