package geniusweb.protocol.session.saop;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.protocol.session.TeamOfPartiesAndProfiles;
import geniusweb.references.Parameters;
import geniusweb.references.PartyRef;
import geniusweb.references.PartyWithParameters;
import geniusweb.references.ProfileRef;
import tudelft.utilities.junit.GeneralTests;

public class SaopPartyWithProfileTest
		extends GeneralTests<SaopPartyWithProfile> {

	private PartyRef party1, party1a, party2, party3;
	private ProfileRef profile1, profile2;
	private SaopPartyWithProfile partyprof1, partyprof1a, partyprof2,
			partyprof3, partyprof4;
	private final String serialized = "{\"party\":{\"partyref\":\"ws:party1\",\"parameters\":{}},\"profile\":\"ws:profile1\"}";

	@Before
	public void before() throws URISyntaxException {
		party1 = new PartyRef(new URI("ws:party1"));
		party1a = new PartyRef(new URI("ws:party1"));
		party2 = new PartyRef(new URI("ws:party2"));
		party3 = new PartyRef(new URI("http:party3"));

		profile1 = new ProfileRef(new URI("ws:profile1"));
		profile2 = new ProfileRef(new URI("ws:profile2"));

		Parameters settings1 = new Parameters();
		Parameters settings2 = new Parameters();
		settings2.put("a", 1);

		PartyWithParameters party1withparams1 = new PartyWithParameters(party1,
				settings1);
		PartyWithParameters party1withparams2 = new PartyWithParameters(party1,
				settings2);
		PartyWithParameters party2withparams1 = new PartyWithParameters(party2,
				settings1);

		partyprof1 = new SaopPartyWithProfile(party1withparams1, profile1);
		partyprof1a = new SaopPartyWithProfile(party1withparams1, profile1);
		partyprof2 = new SaopPartyWithProfile(party2withparams1, profile1);
		partyprof3 = new SaopPartyWithProfile(party1withparams1, profile2);
		partyprof4 = new SaopPartyWithProfile(party1withparams2, profile1);

	}

	@Override
	public List<List<SaopPartyWithProfile>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(partyprof1, partyprof1a),
				Arrays.asList(partyprof2), Arrays.asList(partyprof3),
				Arrays.asList(partyprof4));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"PartyWithProfile.PartyRef.ws:party1.,ProfileRef.ws:profile1.*",
				"PartyWithProfile.PartyRef.ws:party2.,ProfileRef.ws:profile1.*",
				"PartyWithProfile.PartyRef.ws:party1.,ProfileRef.ws:profile2.*",
				"PartyWithProfile.PartyRef.ws:party1.\\{a=1\\},ProfileRef.ws:profile1.*");
	}

	@Test
	public void smokeTest() throws URISyntaxException {
	}

	@Test(expected = IllegalArgumentException.class)
	public void nullTest() throws URISyntaxException {
		new SaopPartyWithProfile((PartyWithParameters) null, (ProfileRef) null);
	}

	@Test
	public void testSerialize() throws JsonProcessingException {
		ObjectMapper jackson = new ObjectMapper();

		String json = jackson.writeValueAsString(partyprof1);
		System.out.println(json);
		assertEquals(serialized, json);
	}

	@Test
	public void testDeserialize()
			throws JsonParseException, JsonMappingException, IOException {
		ObjectMapper jackson = new ObjectMapper();
		// always used as part of bigger class, do not include class name
		// and therfore can not serialize as TeamOfPartiesAndProfiles
		TeamOfPartiesAndProfiles p = jackson.readValue(serialized,
				SaopPartyWithProfile.class);
		System.out.println(p);
		assertEquals(partyprof1, p);
	}

}
